<?php
/**
 * @link https://cms.skeeks.com/
 * @copyright Copyright (c) 2010 SkeekS
 * @license https://cms.skeeks.com/license/
 * @author Semenov Alexander <semenov@skeeks.com>
 */

namespace skeeks\cms\components\storage;

use skeeks\cms\models\StorageFile;
use skeeks\sx\Dir;
use skeeks\sx\File;

/**
 * @author Semenov Alexander <semenov@skeeks.com>
 */
class ClusterLocal extends Cluster
{
    /**
     * @var bool
     */
    public $publicBaseUrlIsAbsolute = false;

    public $hostInfo = "";

    public function init()
    {
        if (!$this->name) {
            $this->name = \Yii::t('skeeks/cms', "Local storage");
        }

        if (!$this->hostInfo) {
            $this->hostInfo = \Yii::$app->urlManager->hostInfo;
        }

        if (!$this->publicBaseUrl) {
            $this->publicBaseUrl = \Yii::getAlias("@web/uploads/all");
        } else {
            $this->publicBaseUrl = \Yii::getAlias($this->publicBaseUrl);
        }

        if (!$this->rootBasePath) {
            $this->rootBasePath = \Yii::getAlias("@webroot/uploads/all");
        } else {
            $this->rootBasePath = \Yii::getAlias($this->rootBasePath);
        }
        
        parent::init();
    }
    /**
     * Добавление файла в кластер
     *
     * @param File $tmpFile
     * @return string
     * @throws Exception
     */
    public function upload(File $tmpFile)
    {
        $clusterFileName = $this->_generateClusterFileName($tmpFile);

        $dir = $this->rootBasePath;
        $localPath = $this->getClusterDir($clusterFileName);

        $clusterFileSrc = $clusterFileName;

        if ($localPath) {
            $clusterFileSrc = $localPath.DIRECTORY_SEPARATOR.$clusterFileSrc;
        }

        try {
            $dir = new Dir($dir.DIRECTORY_SEPARATOR.$localPath);
            $resultFile = $dir->newFile($clusterFileName);
            $tmpFile->move($resultFile);

        } catch (\yii\base\Exception $e) {
            throw new \yii\base\Exception($e->getMessage());
        }

        return $clusterFileSrc;
    }

    /**
     * Удаление файла
     *
     * @param $clusterFileSrc
     * @return bool
     * @throws Exception
     */
    public function delete(StorageFile $clusterFile)
    {
        if (!$clusterFile->cluster_file) {
            //Если не задан путь к файлу, то не нужно удлаять, может удалиться все хранилище
            return true;
        }

        $file = new File($this->getRootSrc($clusterFile));
        if ($file->isExist()) {
            $file->remove();
        }

        return true;
    }


    /**
     * Удаление временной папки
     *
     * @param $clusterFileUniqSrc
     * @return bool|mixed
     */
    public function deleteTmpDir(StorageFile $clusterFile)
    {
        if (!$clusterFile->cluster_file) {
            //Если не задан путь к файлу, то не нужно удлаять, может удалиться все хранилище
            return true;
        }

        if ($this->rootBasePath == $this->rootTmpDir($clusterFile)) {
            //Если не задан путь к файлу, то не нужно удлаять, может удалиться все хранилище
            return true;
        }

        $dir = new Dir($this->rootTmpDir($clusterFile), false);
        if ($dir->isExist()) {
            $dir->remove();
        }

        return true;
    }

    public function update($clusterFileUniqSrc, $file)
    {
    }

    /**
     * @param $clusterFileUniqSrc
     * @return string
     */
    public function getAbsoluteUrl(StorageFile $clusterFile)
    {
        if ($this->publicBaseUrlIsAbsolute) {
            return $this->getPublicUrl($clusterFile);
        } else {
            return $this->hostInfo . $this->getPublicUrl($clusterFile);
        }
    }
    
    /**
     * @param $clusterFileUniqSrc
     * @return string
     */
    public function getAbsoluteBaseNameUrl(StorageFile $clusterFile)
    {
        if ($this->publicBaseUrlIsAbsolute) {
            return $this->getPublicBaseNameUrl($clusterFile);
        } else {
            return $this->hostInfo . $this->getPublicBaseNameUrl($clusterFile);
        }
    }
    
    /**
     * Свободное место на сервере
     * @return float
     */
    public function getFreeSpace()
    {
        if ($this->existsRootPath()) {
            return (float)disk_free_space($this->rootBasePath);
        }

        return (float)0;
    }
    /**
     * @return bool
     */
    public function existsRootPath()
    {
        if (is_dir($this->rootBasePath)) {
            return true;
        }

        //Создать папку для файлов если ее нет
        $dir = new Dir($this->rootBasePath);
        if ($dir->make()) {
            return true;
        }

        return false;
    }
    /**
     * Всего столько места.
     * @return float
     */
    public function getTotalSpace()
    {
        if ($this->existsRootPath()) {
            return (float)disk_total_space($this->rootBasePath);
        }

        return (float)0;
    }
}