<?php
/**
 * Модель значения связанного свойства.
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010 SkeekS (СкикС)
 * @date 18.05.2015
 */

namespace skeeks\cms\relatedProperties\models;

use skeeks\cms\models\Core;
use Yii;

/**
 * @property integer $id
 * @property integer $created_by
 * @property integer $updated_by
 * @property integer $created_at
 * @property integer $updated_at
 * @property integer $property_id
 * @property string $value
 * @property string $def
 * @property string $code
 * @property integer $priority
 *
 * @property RelatedPropertyModel $property
 */
abstract class RelatedPropertyEnumModel extends Core
{
    /**
     * @inheritdoc
     */
    public function attributeHints()
    {
        return array_merge(parent::attributeHints(), [
            'code' => "Не обязательно к заполнению. Это поле будет сгенерировано автоматически.",
        ]);
    }
    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return array_merge(parent::attributeLabels(), [
            'id' => Yii::t('skeeks/cms', 'ID'),
            'created_by' => Yii::t('skeeks/cms', 'Created By'),
            'updated_by' => Yii::t('skeeks/cms', 'Updated By'),
            'created_at' => Yii::t('skeeks/cms', 'Created At'),
            'updated_at' => Yii::t('skeeks/cms', 'Updated At'),
            'property_id' => Yii::t('skeeks/cms', 'Property'),
            'value' => Yii::t('skeeks/cms', 'Value'),
            'def' => Yii::t('skeeks/cms', 'Default'),
            'code' => Yii::t('skeeks/cms', 'Code'),
            'priority' => Yii::t('skeeks/cms', 'Priority'),
        ]);
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return array_merge(parent::rules(), [
            [['created_by', 'updated_by', 'created_at', 'updated_at', 'property_id', 'priority'], 'integer'],
            [['value', 'property_id'], 'required'],
            [['value'], 'string', 'max' => 255],
            [['def'], 'string', 'max' => 1],
            [['code'], 'string', 'max' => 32],
            [
                'code',
                'default',
                'value' => function($model, $attribute) {
                    return md5(rand(1, 10) . time());
                }
            ],
            [
                'priority',
                'default',
                'value' => function($model, $attribute) {
                    return 500;
                }
            ],
        ]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    abstract public function getProperty();
    /*{
        return $this->hasOne(CmsContentProperty::className(), ['id' => 'property_id']);
    }*/

    public function asText()
    {
        $text = parent::asText();
        $text .= $this->value;
        return $text;
    }
}